package yunzmall

import (
	"context"
	"github.com/gogf/gf/encoding/gjson"
	"github.com/gogf/gf/frame/g"
	"github.com/gogf/gf/util/gconv"
)

type refundLogic struct {
}

var Refund = refundLogic{}

type RefundBeforeRes struct {
	CommonRes
	Data map[string]string `json:"data"`
}

func (s refundLogic) Before(ctx context.Context, req string) (res *RefundBeforeRes, err error) {
	var method = "/app/afterSales/getAfterSalesTypeNameMap"
	result, err := post(ctx, method, g.Map{
		"order_item_id": req,
	})
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

type RefundDetailRes struct {
	CommonRes
	Data struct {
		Id           int      `json:"id"`
		Contacts     string   `json:"contacts"`
		Tel          string   `json:"tel"`
		Plane        string   `json:"plane"`
		Country      string   `json:"country"`
		Province     int      `json:"province"`
		City         int      `json:"city"`
		District     int      `json:"district"`
		Address      string   `json:"address"`
		AddressType  []string `json:"address_type"`
		IsDefault    int      `json:"is_default"`
		SupplierId   int      `json:"supplier_id"`
		ProvinceName struct {
			Id       int    `json:"id"`
			ParentId int    `json:"parent_id"`
			Name     string `json:"name"`
			Level    int    `json:"level"`
		} `json:"province_name"`
		CityName struct {
			Id       int    `json:"id"`
			ParentId int    `json:"parent_id"`
			Name     string `json:"name"`
			Level    int    `json:"level"`
		} `json:"city_name"`
		DistrictName struct {
			Id       int    `json:"id"`
			ParentId int    `json:"parent_id"`
			Name     string `json:"name"`
			Level    int    `json:"level"`
		} `json:"district_name"`
		Supplier struct {
			Id       int    `json:"id"`
			Name     string `json:"name"`
			Supplier string `json:"supplier"`
		} `json:"supplier"`
	} `json:"data"`
}

// Detail 详情
func (s refundLogic) Detail(ctx context.Context, req string) (res *RefundDetailRes, err error) {
	var method = "/app/supplier/findShopAddressByOrderId"
	result, err := post(ctx, method, g.Map{
		"after_sales_id": req,
	})
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

// Reason 售后原因
func (s refundLogic) Reason(ctx context.Context, afterSaleType, isReceived int) (res *RefundDetailRes, err error) {
	var method = "/app/afterSales/reason/list"
	result, err := post(ctx, method, g.Map{
		"after_sale_type": afterSaleType,
		"is_received":     isReceived,
	})
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

type RefundCreateReq struct {
	OrderId      int      `json:"order_id,omitempty"`       //订单id
	ThirdOrderSn string   `json:"third_order_sn,omitempty"` //采购端订单号
	OrderItemId  int      `json:"order_item_id"`            //子订单id
	ReasonType   int      `json:"reason_type"`              //售后原因
	Description  string   `json:"description"`              //描述
	Reason       string   `json:"reason"`                   //售后原因是 其他的的时候填写
	RefundType   int      `json:"refund_type"`              //退款还是退货退款
	IsReceived   int      `json:"is_received"`              //是否收到货 1是0否
	DetailImages []string `json:"detail_images,omitempty"`  //图片凭证
	RefundWay    int      `json:"refund_way,omitempty"`     //退款方式
	Num          int      `json:"num"`                      //退款商品个数
	BarterSkuId  int      `json:"barter_sku_id,omitempty"`  //换货中台规格id
	BarterNum    int      `json:"barter_num,omitempty"`     //换货数量
}

type RefundCreateRes struct {
	CommonRes
	Data int `json:"data"`
}

// Create 申请售后
func (s refundLogic) Create(ctx context.Context, req RefundCreateReq) (res *RefundCreateRes, err error) {
	var method = "/app/afterSales/create"
	result, err := post(ctx, method, gconv.Map(req))
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}
